<?php

defined( '_JEXEC' ) or die( 'Restricted access' );

/**
 * @version 2.0 $Id: ThumbnailCreator.php,v 1.3 2011-05-25 19:59:21 harry Exp $
 * @package Joomla
 * @subpackage H2N Mosimage Plugin
 * @copyright (C) 2010 Harry Klein - www.joomla-hklein.de
 * @copyright Copyright (C) 2005 Open Source Matters. All rights reserved.
 * @license GNU/GPL, see LICENSE.php
 *
 * H2N Mosimage Plugin is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 2
 * as published by the Free Software Foundation.
 *
 * H2N Mosimage Plugin is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with EventList; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

require_once JPATH_ROOT . DS.'plugins'.DS.'content'.DS.'mosimage'.DS.'helper'.DS.'MosimageProperties.php';

class ThumbnailCreator {
	
	public function __construct(){
	
	}
	
	
	
	
	/** liefert Dateinamen für des Vorschau- bzw. Vollbild 
	 *
	 * @param String $thumb_file
	 * @param String $ext
	 * @param String $scramble md5,crc32 oder leer
	 * @return string|null*/
	
	static function calcFilename($thumb_file, $ext, $scramble){
		switch($scramble) {
			case 'md5': 
				return md5($thumb_file) . '.' . $ext; 
				break;
			case 'crc32': 
				return sprintf("%u", crc32($thumb_file)) . '.' . $ext;
				break;
			default:
				return $thumb_file;
		}
	}
	
	/**
	 * 
	 * Erzeugt das PlaceholderImage und liefert dessen URL, Höhe und Breite. 
	 * @param int $width
	 * @param int $height
	 * @param String $scramble 
	 * @return MosimageProperties
	 */
	private function generatePlaceHolder($width, $height, $scramble){
		$document=& JFactory::getDocument();
		$language = $document->getLanguage();
		$thumb_file = $this->calcFilename("$width.$height.$language.placeholder.jpg", 'jpg', $scramble);
		$thumbname =  $this->getCachePath() . DS  . $thumb_file;
		if(file_exists($thumbname)) {
			$size = @getimagesize($thumbname);
			if($size) {
				$width = $size[0];
				$height = $size[1];
				return new MosimageProperties($this->buildUrlToImage($thumbname), $width, $height);
			}
		}
		$placeholderImage = ImageCreateTrueColor($width, $height);
		imagefilledrectangle($placeholderImage, 2, 2, $width-3, $height-3, 0xeeeeee);
		$fontSize = 2;
		$string = JText::_('generate image');
		$textWidth = imagefontwidth($fontSize)*strlen($string);
		$center = ceil($width / 2);	
		$x = $center - (ceil($textWidth/2));
		$color = ImageColorAllocate ($placeholderImage, 0, 0, 0); 
		ImageString($placeholderImage, $fontSize, $x, 45, $string, $color); 
		$result = imagejpeg($placeholderImage, $thumbname, 80);	
		imagedestroy($placeholderImage);
		return new MosimageProperties($this->buildUrlToImage($thumbname), $width, $height);
	}
	
	public function createThumbnailImage(&$config, &$absoluteFileName, &$thumb_width, &$thumb_heigth){
		return $this->createThumbnail($config, $absoluteFileName, $thumb_width, $thumb_heigth);
	}
	
	public function createResizedImage(&$config, &$absoluteFileName, &$width, &$height ){
		$watermarkfile = JPATH_SITE .DS.'images' . DS .'stories'. DS . $config->getWatermarkFile();
		return $this->createThumbnail($config, $absoluteFileName, $width, $height, true , 0xFFFFFF, $watermarkfile);
	}
	
	private function createThumbnail(&$config, $filename, $width, $height, $chageProportions=false , $bgcolor = 0xFFFFFF, $watermarkfile = '') {
		static $countGenerateImage;
		
		if (!isset($countGenerateImage)){
			$countGenerateImage = 0;
		}
		static $disablegifwarning, $disablepngwarning, $disablejpgwarning, $disablepermissionwarning;
		
		$cachepath = $this->getCachePath(); 
		if (!JFolder::exists($cachepath)){
			JFolder::create($cachepath);
		}
	
		$temp = pathinfo($filename);
		$ext = $temp['extension']; //todo: check filename here
		$alt_filename = '';
	
		$size = @getimagesize($filename);
		if(!$size) {
			JError::raiseWarning(200,"There was a problem loading image [$filename]");
			return false;
		}
		$origw = $size[0];
		$origh = $size[1];
		if($alt_filename && ($origw<$width || $origh<$height)) {
			$width = $origw;
			$height = $origh;
		}
		
		if ($config->isUsedWatermark() && $watermarkfile){
			$watermark = 1;
		} else {
			$watermark = 0;
		}
		if ($chageProportions) {
			$proportion = $config->getImageProportions();
		}
		else {
			$proportion='bestfit';
		}
		if($origw<$width && $origh<$height) {
			$proportion='fill';
		}
		
		if($width || $height) {
			$prefix = substr($proportion,0,1) . ".$width.$height.$bgcolor.$watermark.";
		}
		else {
			$prefix = "$watermark.";
		}
		$thumb_file = $prefix . str_replace(array(':','/','\\'),  '.' ,substr($filename,strlen(JPATH_SITE)+8));
		$thumb_file = ThumbnailCreator::calcFilename($thumb_file, $ext, $config->getScrambleFilename());
		$thumbname =  $cachepath . DS  . $thumb_file;
		if(file_exists($thumbname))
		{
			$size = @getimagesize($thumbname);
			if($size) {
				$width = $size[0];
				$height = $size[1];
			}
		}
		else {		
			if(!($width || $height)) { // Both sides zero
				$width = $origw;
				$height = $origh;
				$proportion='stretch';
			}
			elseif(!($width && $height)){ // One of the sides zero
				$proportion = 'bestfit';
			}
			$newheight = $height; $newwidth = $width;
			$dst_x = $dst_y = $src_x = $src_y = 0;
			if((($origw < $origh && $width < $height) || ($origw >= $origh && $width <= $height)) && $width != 0 || $height == 0)
			switch($proportion) {
				case 'fill':
					$newheight =  $origh/$origw*$width;
					$dst_y = ($height - $newheight)/2;
					break;
				case 'crop':
					$temph = $origw;
					$origw = $origh / $newheight * $newwidth;
					$src_x = ($temph-$origw)/2;
					break;
				case 'bestfit':
					$newheight = $height = round($width / ($origw / $origh));
			}
			else
			switch($proportion) {
				case 'fill':
					$newwidth =  $origw/$origh*$height;
					$dst_x = ($width - $newwidth)/2;
					break;
				case 'crop':
					$temph = $origh;
					$origh = $origw * $newheight / $newwidth;
					$src_y = ($temph-$origh)/2;
					break;
				case 'bestfit':
					$newwidth = $width = round($height * ($origw / $origh));
			}
			if ($countGenerateImage >  $config->getMaxResizedImagePerRequest()) {
				return $this->generatePlaceHolder($width, $height, $config->getScrambleFilename());
			}
			$countGenerateImage++;
			
			switch(strtolower($ext)) {
				case 'png':
					if(!function_exists("imagecreatefrompng")) {
						if(!$disablepngwarning) {
							JError::raiseWarning(200,'PNG is not supported on this server.');
						}
						$disablepngwarning = true;
						return false;
					}
					else {
						$src_img = imagecreatefrompng($filename);
						$imagefunction = "imagepng";
					}
					break;
				case 'gif':
					if(!function_exists("imagecreatefromgif")) {
						if(!$disablegifwarning) {
							JError::raiseWarning(200,'GIF is not supported on this server.');
						}
						$disablegifwarning = true;
						return false;
					}
					else {
						$src_img = imagecreatefromgif($filename);
						$imagefunction = "imagegif";
					}
					break;
				default:
					if(!function_exists("imagecreatefromjpeg")) {
						if(!$disablejpgwarning) {
							JError::raiseWarning(200,'JPG is not supported on this server.');
						}
						$disablejpgwarning = true;
						return false;
					}
					else {
						$src_img = imagecreatefromjpeg($filename);
						$imagefunction = "imagejpeg";
					}
			}
			$dst_img = ImageCreateTrueColor($width, $height);
			imagefill( $dst_img, 0,0, $bgcolor);
			imagecopyresampled($dst_img,$src_img, $dst_x, $dst_y, $src_x, $src_y, $newwidth,$newheight,$origw, $origh);
	
			if($watermarkfile && $watermark) {
				if($config->getWatermarkTransparencyType() == 'alpha') {
					$transcolor = FALSE;
				}
				else {
					$transcolor = $config->getWatermarkTransparentColor();
				}
				$this->createWatermark($dst_img, $watermarkfile, $config->getWatermarkLeft(), $config->getWatermarkTop(), $transcolor, $config-> getWatermarkTransparency());
			}
			if($imagefunction=='imagejpeg') {
				$result = @$imagefunction($dst_img, $thumbname, $config->getJpegQuality());
			}
			else {
				$result = @$imagefunction($dst_img, $thumbname);
			}
			imagedestroy($src_img);
			if(!$result) {
				if(!$disablepermissionwarning) {
					JError::raiseWarning(200,"Could not create image: [$thumbname]. Check if you have write permissions in ·[$cachepath]");
				}
				$disablepermissionwarning = true;
			}
			else
			imagedestroy($dst_img);
		}
		$url = JRoute::_($this->buildUrlToImage($thumbname));
		return new MosimageProperties($url, $width, $height);
	}	
	
	private function createWatermark($sourcefile_id, $watermarkfile, $x, $y, $transcol = false, $transparency = 100) {
		static $disable_wm_ext_warning, $disable_wm_load_warning, $disable_alpha_warning;
	
		//Get the resource ids of the pictures
		$fileType = strtolower(substr($watermarkfile, strlen($watermarkfile)-3));
		switch($fileType) {
			case 'png':
				$watermarkfile_id = @imagecreatefrompng($watermarkfile);
				break;
			case 'gif':
				$watermarkfile_id = @imagecreatefromgif($watermarkfile);
				break;
			case 'jpg':
				$watermarkfile_id = @imagecreatefromjpeg($watermarkfile);
				break;
			default:
				$watermarkfile = basename($watermarkfile);
				if(!$disable_wm_ext_warning) {
					JError::raiseWarning(200,"You cannot use a [.$fileType] file ($watermarkfile) as a watermark");
				}
				$disable_wm_ext_warning = true;
				return false;
		}
		if(!$watermarkfile_id) {
			if(!$disable_wm_load_warning) {
				JError::raiseWarning(200,"There was a problem loading the watermark [$watermarkfile]");
			}
			$disable_wm_load_warning = true;
			return false;
		}
		@imageAlphaBlending($watermarkfile_id, false);
		$result = @imageSaveAlpha($watermarkfile_id, true);
		if(!$result) {
			if(!$disable_alpha_warning) {
				JError::raiseWarning(200, "Watermark problem: your server does not support alpha blending (requires GD 2.0.1+)");
			}
			$disable_alpha_warning = true;
			imagedestroy($watermarkfile_id);
			return false;
		}
		//Get the sizes of both pix
		$sourcefile_width=imageSX($sourcefile_id);
		$sourcefile_height=imageSY($sourcefile_id);
		$watermarkfile_width=imageSX($watermarkfile_id);
		$watermarkfile_height=imageSY($watermarkfile_id);
		if(!$x) {
			$dest_x = ( $sourcefile_width / 2 ) - ( $watermarkfile_width / 2 );
		}
		elseif($x>0) {
			$dest_x = $x;
		}
		else {
			$dest_x = $sourcefile_width - $watermarkfile_width + $x;
		}
		if(!$y) {
			$dest_y = ( $sourcefile_height / 2 ) - ( $watermarkfile_height / 2 );
		}
		elseif($y>0) {
			$dest_y = $y;
		}
		else {
			$dest_y = $sourcefile_height - $watermarkfile_height + $y;
		}
		// if a gif, we have to upsample it to a truecolor image
		if($fileType == 'gif') {
			// create an empty truecolor container
			$tempimage = imagecreatetruecolor($sourcefile_width, $sourcefile_height);
				
			// copy the 8-bit gif into the truecolor image
			imagecopy($tempimage, $sourcefile_id, 0, 0, 0, 0, $sourcefile_width, $sourcefile_height);
				
			// copy the source_id int
			$sourcefile_id = $tempimage;
		}
		if($transcol!==false) {
			imagecolortransparent($watermarkfile_id, $transcol); // use transparent color
			imagecopymerge($sourcefile_id, $watermarkfile_id, $dest_x, $dest_y, 0, 0, $watermarkfile_width, $watermarkfile_height, $transparency);
		}
		else {
			imagecopy($sourcefile_id, $watermarkfile_id, $dest_x, $dest_y, 0, 0, $watermarkfile_width, $watermarkfile_height); // True alphablend
		}
		imagedestroy($watermarkfile_id);
	}
	
	
	private function getRelativeCachePath(){
		return 'cache' . DS . 'mosimage-cache';
	}
	
	private function getCachePath(){
		return JPATH_SITE. DS . $this->getRelativeCachePath();
	}
	
	private function buildUrlToImage($thumbname){
		return JURI::base(). '/' . $this->getRelativeCachePath() . '/' . rawurlencode(basename($thumbname));
	}	
}


?>